searchNodes=[{"type":"module","title":"Bindable","doc":"Elixir for-comprehension that goes beyond the List s. iex&gt; import Bindable.ForComprehension ...&gt; import Bindable.Maybe ...&gt; ...&gt; bindable for x &lt;- just ( 1 ) , ...&gt; y &lt;- just ( 2 ) , ...&gt; x + y &gt; 4 , ...&gt; z &lt;- just ( 3 ) , ...&gt; do : x + y + z nothing ( ) For-comprehension is a common syntax construct in functional programming languages, that allows you to express complex operations on collections and monadic contexts in a concise and expressive way. More formally, it is a way to construct monadic value by describing it as a sequence of effectful computations. To do so, it provides following constructs to combine such computations into new monadic value: Generator (commonly known as iterator for lists): extracts value from the monadic context. Guard (or filter): filters generated values based on a predicate (see Bindable.Empty ). Assign: aliases any expression inside current scope. Yield: defines resulting value to return inside the monadic context. Here comes an example of them: iex&gt; import Bindable.ForComprehension ...&gt; ...&gt; xs = [ [ 10 , 20 ] , [ 30 ] ] ...&gt; bindable for x &lt;- xs , # generator ...&gt; length ( x ) &gt; 1 , # guard ...&gt; y &lt;- x , # next generator ...&gt; z = y + 1 , # assign ...&gt; y + z &gt; 21 , # another guard ...&gt; do : { y , z } # yield [ { 20 , 21 } ] Elixir's kernel provides for-comprehension only for lists. It works with any Enumerable , however it always (eagerly) yields List (so you can't describe Stream using Kernel.SpecialForms.for/1 ). Bindable already comes with for-comprehension batteries for: List Bindable.Maybe Stream So you can lazily create new stream out of provided ones: iex&gt; import Bindable.ForComprehension ...&gt; ...&gt; lazy_xs = Stream . take ( 1 .. 5 , 2 ) ...&gt; lazy_ys = Stream . take ( 5 .. 9 , 2 ) ...&gt; lazy_xys = bindable for x &lt;- lazy_xs , ...&gt; y &lt;- lazy_ys , ...&gt; do : { x , y } ...&gt; ...&gt; { is_list ( lazy_xys ) , Enum . to_list ( lazy_xys ) } { false , [ { 1 , 5 } , { 1 , 6 } , { 2 , 5 } , { 2 , 6 } ] } The main goal of the library is to provide for-comprehension beyond lists with the least amount of overhead. So it doesn't aim to provide a principled way to define type classes (required by for-context), e.g. it doesn't provide any sensible way to enforce type class properties on implementations. To stick with &quot;the least amount of overhead&quot; paradigm type classes implemented atop of Elixir protocols. &quot;Minimal complete definition&quot; for you data type to be compliant with for-comprehension includes: Bindable.FlatMap implementation to chain sequential generators; Bindable.Pure implementation to yield resulting value (design decision, see Bindable.ForComprehension ). If you want to use guards/filters inside for-comprehension with your data type, you should also provide an implementation for Bindable.Empty , so it is optional, e.g. when your data type does not provide any meaningful semantics for empty/filtered value effect.","ref":"Bindable.html"},{"type":"protocol","title":"Bindable.Empty","doc":"&quot;Type class&quot; to represent a failed or empty computation. Informally, it defines &quot;filtering&quot; semantics for your type. By implementing it you enable guards inside for-comprehension for your type. Inspired by Alternative 's empty .","ref":"Bindable.Empty.html"},{"type":"function","title":"Bindable.Empty.of/1","doc":"Provides a default or fallback value in cases where a computation fails or has no result. Returns an &quot;empty&quot;-value of the type provided by example .","ref":"Bindable.Empty.html#of/1"},{"type":"type","title":"Bindable.Empty.t/0","doc":"All the types that implement this protocol.","ref":"Bindable.Empty.html#t:t/0"},{"type":"protocol","title":"Bindable.FlatMap","doc":"&quot;Type class&quot; to chain together computations that have a context or effect. Required to enable for-comprehension for your type. Inspired by Monad 's bind (also known as flatMap or &gt;&gt;= ).","ref":"Bindable.FlatMap.html"},{"type":"function","title":"Bindable.FlatMap.flat_map/2","doc":"Takes a computation that produces a value of type a and a function, that takes that value and produces a new computation, that produces a value of type b . The key feature is that it allows the function to access the value produced by the first computation and use it to construct the second computation (in contrast with Applicative 's &lt;|&gt; ).","ref":"Bindable.FlatMap.html#flat_map/2"},{"type":"type","title":"Bindable.FlatMap.t/0","doc":"All the types that implement this protocol.","ref":"Bindable.FlatMap.html#t:t/0"},{"type":"module","title":"Bindable.ForComprehension","doc":"Elixir for-comprehension that goes beyond the List s. For-comprehension (its operations) could be implemented in several ways. For example, you can desugar it into chain of flatMap + map invocation: iex&gt; xs = [ 1 , 2 ] ...&gt; ys = [ 3 , 4 ] ...&gt; for ( x &lt;- xs , y &lt;- ys , do : { x , y } ) === Enum . flat_map ( xs , fn x -&gt; Enum . map ( ys , fn y -&gt; { x , y } end ) end ) true But if you try to implement assigns using flatMap + map , you will end up with doing something like map(ma, &amp;{&amp;1, assign}) , and then each guard, that doesn't utilize every variable of such tuple, would raise an unused variable compiler warning ( filter(ma, fn {a, b} -&gt; a &gt; 42 end) ). However, suppressing all these warning would be a bad idea. They are useful to detect several kinds of flaws, while working with for-comprehension. For example inefficient guards: iex&gt; import Bindable.ForComprehension ...&gt; bindable for x &lt;- [ 1 , 2 ] , y &lt;- [ 3 , 4 ] , x &lt; 2 , do : { x , y } [ { 1 , 3 } , { 1 , 4 } ] In this case the guard does not refer to the last generated value, so the expression can be refactored to apply the guard before the last generator. It can dramatically improve performance of the whole expression in general! iex&gt; import Bindable.ForComprehension ...&gt; bindable for x &lt;- [ 1 , 2 ] , x &lt; 2 , y &lt;- [ 3 , 4 ] , do : { x , y } [ { 1 , 3 } , { 1 , 4 } ] That is why flatMap + pure was used for implementation, so assigns could be implemented in a most straightforward way, while preserving all compiler warnings. Compiler would warn you (about unused variable), if you try to apply the guard, which does not refer to the last generated value.","ref":"Bindable.ForComprehension.html"},{"type":"macro","title":"Bindable.ForComprehension.bindable/1","doc":"Elixir for-comprehension that goes beyond the List s. Elixir does not have variadic functions or macros. But Elixir's for-comprehension looks exactly like variadic macro: iex&gt; for ( x &lt;- [ 1 , 2 ] , y &lt;- [ 3 , 4 ] , do : { x , y } ) [ { 1 , 3 } , { 1 , 4 } , { 2 , 3 } , { 2 , 4 } ] That's because it is actually a Kernel.SpecialForms.for/1 , treated &quot;specially&quot; by compiler. So to emulate variadic nature of for-comprehension (e.g. it can have one or many generators) macro application was used (macro applied to Kernel.SpecialForms.for/1 expression): iex&gt; import Bindable.ForComprehension ...&gt; bindable for x &lt;- [ 1 , 2 ] , y &lt;- [ 3 , 4 ] , do : { x , y } [ { 1 , 3 } , { 1 , 4 } , { 2 , 3 } , { 2 , 4 } ] It also supports Kernel.SpecialForms.for/1 -like guards and assigns.","ref":"Bindable.ForComprehension.html#bindable/1"},{"type":"macro","title":"Bindable.ForComprehension.bindable/2","doc":"","ref":"Bindable.ForComprehension.html#bindable/2"},{"type":"macro","title":"Bindable.ForComprehension.do_for/5","doc":"&quot;Private&quot; macro to implement for/2 . Due to Elixir's macro expand process nature you can't have this do_for macro as a &quot;private&quot; one. It would require do_for import at the call-site, which is impossible for &quot;private&quot;-macro (defined by defmacrop ).","ref":"Bindable.ForComprehension.html#do_for/5"},{"type":"module","title":"Bindable.Maybe","doc":"Yet another Maybe implementation. Maybe is a data type that is used to represent optional values that may or may not be present. It is typically used as a safer alternative to using null or undefined values.","ref":"Bindable.Maybe.html"},{"type":"function","title":"Bindable.Maybe.just/1","doc":"","ref":"Bindable.Maybe.html#just/1"},{"type":"function","title":"Bindable.Maybe.nothing/0","doc":"","ref":"Bindable.Maybe.html#nothing/0"},{"type":"function","title":"Bindable.Maybe.of_nullable/1","doc":"","ref":"Bindable.Maybe.html#of_nullable/1"},{"type":"type","title":"Bindable.Maybe.just/1","doc":"","ref":"Bindable.Maybe.html#t:just/1"},{"type":"type","title":"Bindable.Maybe.nothing/0","doc":"","ref":"Bindable.Maybe.html#t:nothing/0"},{"type":"type","title":"Bindable.Maybe.t/1","doc":"","ref":"Bindable.Maybe.html#t:t/1"},{"type":"protocol","title":"Bindable.Pure","doc":"&quot;Type class&quot; to create a computation that has no effect other than to produce a value. Required to enable for-comprehension for your type. Inspired by Applicative 's pure .","ref":"Bindable.Pure.html"},{"type":"function","title":"Bindable.Pure.of/2","doc":"Create a computation that has no effect other than to produce a value. Returns a &quot;pure&quot;-value of the type provided by example .","ref":"Bindable.Pure.html#of/2"},{"type":"type","title":"Bindable.Pure.t/0","doc":"All the types that implement this protocol.","ref":"Bindable.Pure.html#t:t/0"},{"type":"extras","title":"Bindable","doc":"Elixir for-comprehension that goes beyond the List s. import Bindable.ForComprehension import Bindable.Maybe bindable for x &lt;- just ( 1 ) , y &lt;- just ( 2 ) , x + y &gt; 4 , z &lt;- just ( 3 ) , do : x + y + z # nothing()","ref":"readme.html"},{"type":"extras","title":"Bindable - For-comprehension","doc":"For-comprehension is a common syntax construct in functional programming languages, that allows you to express complex operations on collections and monadic contexts in a concise and expressive way. More formally, it is a way to construct monadic value by describing it as a sequence of effectful computations. To do so, it provides following constructs to combine such computations into new monadic value: Generator (commonly known as iterator for lists): extracts value from the monadic context. Guard (or filter): filters generated values based on a predicate (see Bindable.Empty ). Assign: aliases any expression inside current scope. Yield: defines resulting value to return inside the monadic context. import Bindable.ForComprehension xs = [ [ 10 , 20 ] , [ 30 ] ] bindable for x &lt;- xs , # generator length ( x ) &gt; 1 , # guard y &lt;- x , # next generator z = y + 1 , # assign y + z &gt; 21 , # another guard do : { y , z } # yield # [{20, 21}]","ref":"readme.html#for-comprehension"},{"type":"extras","title":"Bindable - Usage","doc":"Elixir's kernel provides for-comprehension only for lists. It works with any Enumerable , however it always (eagerly) yields List (so you can't describe Stream using Kernel.SpecialForms.for/1 ). Bindable already comes with for-comprehension batteries for: List , Maybe , Stream . &quot;Minimal complete definition&quot; for you data type to be compliant with for-comprehension includes: Bindable.FlatMap implementation to chain sequential generators; Bindable.Pure implementation to yield resulting value (design decision, see Bindable.ForComprehension ). If you want to use guards/filters inside for-comprehension with your data type, you should also provide an implementation for Bindable.Empty , so it is optional, e.g. when your data type does not provide any meaningful semantics for empty/filtered value effect. The main goal of the library is to provide for-comprehension beyond lists with the least amount of overhead. So it doesn't aim to provide a principled way to define type classes (required by for-context), e.g. it doesn't provide any sensible way to enforce type class properties on implementations. To stick with &quot;the least amount of overhead&quot; paradigm type classes implemented atop of Elixir protocols.","ref":"readme.html#usage"},{"type":"extras","title":"Bindable - Installation","doc":"The package can be installed by adding bindable to your list of dependencies in mix.exs : def deps do [ { :bindable , &quot;~&gt; 1.0.0&quot; } ] end The docs can be found at https://hexdocs.pm/bindable .","ref":"readme.html#installation"}]